`timescale 1 ps / 1 ps
// omercury 21.05.2015
module SPI_sl	#(parameter W=8)	// W бит slave SPI
(
	input	wire		osc_clk,		//Глобальный клок
//SPI_interface
	input	wire		nSS,			//Вход выбор чипа, асинхронный, подавление метастабильности есть
	input	wire		SCK,			//Вход такта SPI, асинхронный, синхронизация есть
	input	wire		MOSI,			//Вход приёмника, асинхронный, подавление метастабильности есть
	output	wire		MISO,			//Выход передатчика
// Входы-выходы, флаг готовности и стробы
	output	reg	[W-1:0]	SPI_D_out = 0,	//Выход, принятые данные
	input	wire[W-1:0]	SPI_D_in,		//Вход, данных для передачи
	output	reg			ready = 0,		//Выход, готовность приёмника
	input	wire		nRD,			//Вход, строб чтения
	input	wire		nWR				//Вход, строб записи
);
	parameter 			Wcnt	=	2;		//Ширина счетчика входных импульсов (корень из W)
	reg		[Wcnt:0]	counter = 0;		//Счётчик входных импульсов (8 бит)
	reg			[W-1:0]	shiftreg = 0;		//Сдвиговой регистр передатчика
	reg					sDin = 0;			//регистр приёмника
	reg					flag = 0;
// ======================================
assign MISO = shiftreg[W-1];
// ======================================
wire snSS; wire	[1:0]	sync_nSS;
assign sync_nSS[0] = nSS;
assign sync_nSS[1] = sync_nSS[0];
assign snSS = sync_nSS[1] & sync_nSS[0] & nSS;
// ======================================
wire CKplus, CKminus; wire sRD, sWR;					//Внутренние синхронизированные стробы
SincIn	sck1(.Cs(osc_clk), .Ds( SCK), .Qs(CKplus));		//Синхронизация положительного перепада CLOCK
SincIn	sck2(.Cs(osc_clk), .Ds(~SCK), .Qs(CKminus));	//Синхронизация отрицательного перепада CLOCK
SincIn	sck3(.Cs(osc_clk), .Ds(~nRD), .Qs(sRD));		//Синхронизация строба чтения
SincIn	sck4(.Cs(osc_clk), .Ds(~nWR), .Qs(sWR));		//Синхронизация строба записи
// ======================================
always @(posedge osc_clk) begin
	if (snSS) begin					// Сброс
		counter <= 0;							//Обнуление счётчика
		ready <= 1'b0;							//Сброс флага готовности буфера приёма
		flag <= 0;
		shiftreg <= 0;
	end else begin					// Нормальный режим Приёма/Передачи данных
		if (CKplus) begin						//Положительный перепад SCK
			sDin <= MOSI;						//Чтение последовательных данных
			flag <= 1;
		end else if (CKminus && flag) begin		//Отрицательный перепад SCK
			shiftreg[W-1:0] = {shiftreg[W-2:0], sDin};		//Сдвиг передатчика
			flag <= 0;
			if (counter == 3'b111) begin		//Принят восьмой бит
				SPI_D_out <= shiftreg;			//Сохранение приёмника в буфер
				ready <= 1'b1;					//Установка флага готовности буфера приёма
				//shiftreg <= 0;
			end
			counter <= counter + 1;			//Инкремент счётчика
		end else if (sRD && ~sWR)begin			// Чтение
			ready <= 1'b0;						//Сброс флага готовности буфера приёма
		end else if (sWR && ~sRD)begin			// Запись
			shiftreg <= SPI_D_in;				//Загрузка сдвигового регистра
			//shiftreg <= SPI_D_out;				//Загрузка сдвигового регистра
			ready <= 1'b0;						//Сброс флага готовности буфера приёма
		end
	end
end		//always
// ======================================
endmodule 
//**************************************************
//Up level
module SincIn (Cs, Ds, Qs);
input Cs, Ds;
output Qs;
D_Flip	F00(.Cf(Cs), .Df(Ds), .Qf(Q00));
D_Flip	F01(.Cf(Cs), .Df(Q00), .Qf(Q01));
	assign	Qs = (Q00) & (~Q01);
endmodule
//**************************************************
//Down level
// D-FlipFlop with sincronious to Clock
module D_Flip (Cf, Df, Qf);
input Cf, Df;
output Qf;
reg Xf = 0;
assign Qf = Xf;
	always @(posedge Cf) Xf <= Df;
endmodule
//**************************************************